package fr.inra.pappso.gui;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.image.RasterFormatException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import fr.inra.moulon.svgutils.svgspectrum.*;
import fr.inra.pappso.class_pep.Fasts_result;
import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.class_pep.Pep_result;
import fr.inra.pappso.class_pep.Spectre;
import fr.inra.pappso.class_pep.modifs;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pep_exception.StopException;
import fr.inra.pappso.sax_parse.Serialising;

class MyCanvas extends Canvas {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8206876178913623118L;
	Image img;

	public MyCanvas(Image img) {
		this.img = img;
	}

	public void paint(Graphics g) {
		if (img != null) {
			g.drawImage(img, 0, 0, null);
		} else {
			g.clearRect(0, 0, 200, 200);
		}
	}

	public void setImage(Image img) {
		this.img = img;
	}

	public void clear() {
		this.setImage(null);
		this.repaint();
	}
}

public class view_result extends JFrame implements ActionListener,
		TreeSelectionListener {
	private static final Logger logger = Logger.getLogger(view_result.class);

	private static final long serialVersionUID = -5817566283495079782L;

	private String repertoire_in = "";

	private JTextArea align;

	private Ms_run run;

	private JTree tree;

	private MyCanvas svgCanvas = new MyCanvas(null);

	private JPanel pane_canvas;

	public view_result() throws Exception {
		super();
		this.setTitle("View result : ");

		// creation des boutons
		JButton Action1 = new JButton("Loading");
		JButton close = new JButton("Close");
		// creation des actions sur les boutons
		Action1.addActionListener(this);
		close.addActionListener(this);
		// ajout des boutons dans le panel
		JPanel panel = new JPanel();
		panel.add(Action1);
		panel.add(close);

		tree = new JTree(new DefaultTreeModel(new DefaultMutableTreeNode(
				"No selected")));
		tree.getSelectionModel().addTreeSelectionListener(this);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		align = new JTextArea("Alignement");
		align.setFont(new Font("monospaced", Font.PLAIN, 10));

		pane_canvas = new JPanel();
		pane_canvas.setLayout(new BorderLayout());
		pane_canvas.add(svgCanvas, BorderLayout.CENTER);
		JSplitPane splitPane1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true,
				new JScrollPane(tree), pane_canvas);
		splitPane1.setOneTouchExpandable(true);
		Dimension minimumSize = new Dimension(500, 300);
		pane_canvas.setMinimumSize(minimumSize);
		// pane_canvas.setSize(minimumSize);
		// svgCanvas.setMinimumSize(minimumSize);
		// pane_canvas.setSize(minimumSize);

		// svgCanvas.setSize(400, 400);
		// splitPane1.setDividerSize(2);

		JSplitPane splitPane2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				true, new JScrollPane(align), splitPane1);
		splitPane2.setOneTouchExpandable(true);
		// splitPane2.setDividerSize(2);

		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		getContentPane().add(panel, BorderLayout.SOUTH);
		getContentPane().add(splitPane2, BorderLayout.CENTER);
		this.setSize(800, 600);

		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(dim.width / 2 - getWidth() / 2, dim.height / 2
				- getHeight() / 2);

		this.setVisible(true);
		splitPane2.setDividerLocation(0.5);
		splitPane1.setDividerLocation(0.3);
		this.add_tree_data();
	}

	private void add_tree_data() {
		try {
			File fasts = this.get_file_fasts();
			run = new Serialising().unserialise_msrun(fasts.getAbsolutePath());
			DefaultMutableTreeNode root = new DefaultMutableTreeNode();
			Vector<Fasts_result> fasts_results = run.get_Fasts_result();
			int count = 0;
			for (int i = 0; i < fasts_results.size(); i++) {
				Fasts_result fas = fasts_results.elementAt(i);
				if (!fas.isContaminant()) {
					DefaultMutableTreeNode prot = new DefaultMutableTreeNode(
							fas);
					Vector<String> peptides = fas.getscans();
					for (int j = 0; j < peptides.size(); j++) {
						prot.add(new DefaultMutableTreeNode(run
								.get_pepnovo_to_scan(peptides.elementAt(j))));
					}
					root.add(prot);
					count++;
				}
			}
			logger.debug("count " + count);
			TreeModel tm = null;
			if (count > 0) {
				logger.debug("count > 0");
				tm = new DefaultTreeModel(root);
			} else {
				tm = new DefaultTreeModel(new DefaultMutableTreeNode(
						"No selected"));
			}
			tree.setSelectionPath(new TreePath(""));

			tree.setModel(tm);
			this.setTitle("View result : " + run.get_msrun().getName());

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this,
					"Problems during parsing of file\n" + e.getMessage(),
					"Error", JOptionPane.ERROR_MESSAGE);
		}

	}

	private File get_file_fasts() throws StopException {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser
				.setDialogTitle("Select the Fasts result files to be analyzed");
		// fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setCurrentDirectory(new File(repertoire_in));
		my_file_filter FT = new my_file_filter(new String[] { "fasts" },
				"Fasts results file (*.fasts)");
		fileChooser.setFileFilter(FT);
		fileChooser.setPreferredSize(new Dimension(600, 500));
		int returnValue = fileChooser.showOpenDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No Fasts file selected");
		File file = fileChooser.getSelectedFile();
		repertoire_in = "" + fileChooser.getCurrentDirectory();
		return (file);
	}

	private void print_spectra(Pep_result pep, Spectre spectra, boolean b)
			throws PepException {
		// ions ion = new ions();
		// ion.setMods(pep.get_modifs());
		// ion.setSequence(pep.get_Sequence_with_ncter());
		try {
			logger.debug("print_spectra begin");

			SvgPeptideSequence mon_peptide = new SvgPeptideSequence(pep
					.get_Sequence_with_ncter().replaceAll("C", "c"));
			Vector<modifs> mods = pep.get_modifs();
			for (int i = 0; i < mods.size(); i++) {
				modifs mod = mods.elementAt(i);
				mon_peptide.getAA(mod.get_posi()).addModification(
						mod.get_modvalue());
			}

			Dimension v = pane_canvas.getSize();
			logger.debug("SvgSpectrumDocument");
			logger.debug(v.getWidth() + " " + v.getHeight());
			SvgSpectrumDocument test = new SvgSpectrumDocument(
					(float) v.getWidth(), (float) v.getHeight());
			test.setMzIntensity(spectra.get_mass_listes(),
					spectra.get_intensity_liste());
			// test.setIonTable(ion.get_ion_table());
			test.setSequence(mon_peptide);
			test.setFitPrecision(1);
			test.setTitle("Scan : " + pep.get_Scan() + " z=" + pep.get_Charge()
					+ " Sequence : " + pep.get_Sequence_with_ncter());
			if (b) {
				test.toFile(new File("/tmp/pepnovo.svg"));
			}

			Image img = rasterizeSvg(test.getRenderedDocument());
			svgCanvas.setSize(v);
			svgCanvas.setImage(img);
			svgCanvas.repaint();
			// svgCanvas.getGraphics().drawImage(img, 0, 0, null);
			// .setDocument(test.getDocument());
			logger.debug("print_spectra end");
		} catch (Exception e) {
			String message = "error printing spectra :\n" + e.getMessage();
			logger.error(message);
			throw new PepException(message);
		}
	}

	private Image rasterizeSvg(Document document) {
		logger.debug("Entering make png!");
		long duracel = System.currentTimeMillis();
		Image image = null;
		try {
			// this.setIsGelImageEmbedded(false);

			PNGTranscoder jtrans = new PNGTranscoder();
			TranscoderInput input = new TranscoderInput(document);
			ByteArrayOutputStream outPngImage = new ByteArrayOutputStream();
			TranscoderOutput output = new TranscoderOutput(outPngImage);

			logger.info("before");
			jtrans.transcode(input, output);
			logger.info("after");

			// jpegByteArr = outStream.toByteArray();
			outPngImage.flush();
			outPngImage.close();
			image = ImageIO.read(new ByteArrayInputStream(outPngImage
					.toByteArray()));

			ImageIO.write((BufferedImage) image, "png", new File(
					"/tmp/test.png"));

		} catch (RasterFormatException e) {
			logger.error(" RasterFormatException " + e.getMessage());
		} catch (Exception e) {

			logger.error(e.getMessage(), e);
		}
		logger.info("Duration creating PNG: "
				+ (System.currentTimeMillis() - duracel) + " ms");
		return image;
	}

	public void valueChanged(TreeSelectionEvent event) {
		try {
			logger.debug("valueChanged ");
			if (event.getNewLeadSelectionPath() == null) {
				logger.debug("event.getNewLeadSelectionPath() == null");
				align.setText("empty results");
				// svgCanvas.setDocument(null);
				svgCanvas.clear();

				return;
				// throw new PepException("empty results");
			}
			Object[] obj = event.getNewLeadSelectionPath().getPath();
			Fasts_result fas;
			Pep_result pep;
			logger.debug("obj.length " + obj.length);
			if (obj.length == 1) {
				align.setText("No selected");
				// svgCanvas.setDocument(null);
				svgCanvas.clear();
			}
			if (obj.length == 2) {
				fas = (Fasts_result) ((DefaultMutableTreeNode) obj[1])
						.getUserObject();
				align.setText(fas.getResults());
				// svgCanvas.setDocument(null);
				svgCanvas.clear();
			}
			if (obj.length == 3) {
				fas = (Fasts_result) ((DefaultMutableTreeNode) obj[1])
						.getUserObject();
				align.setText(fas.getResults());
				pep = (Pep_result) ((DefaultMutableTreeNode) obj[2])
						.getUserObject();
				print_spectra(pep, run.get_spectra_to_scan(pep.get_Scan()),
						false);

			}
		} catch (Exception e) {
			String message = "Error :\n" + e.getMessage();
			logger.error(message);
			align.setText(message);
			e.printStackTrace();
		} finally {
			repaint();
		}
	}

	public void actionPerformed(ActionEvent evt) {
		String composant = evt.getActionCommand();
		if (composant.equalsIgnoreCase("Loading"))
			add_tree_data();
		if (composant.equalsIgnoreCase("Close"))
			dispose();
	}
}
