package fr.inra.pappso.pipeline;

import java.io.File;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pep_exception.StopException;
import fr.inra.pappso.sax_parse.Serialising;
import fr.inra.pappso.sax_parse.msms_output;
import fr.inra.pappso.shell.fasts_calculation;

public class fasts_pip extends Thread {
	private static final Logger logger = Logger.getLogger(fasts_pip.class);

	fasts_conf conf;

	String basename;

	File pepnovo_seria;

	Ms_run run;

	String status;

	String error;

	boolean stoping;

	public fasts_pip() {
		super();
		status = "Initialising";
		this.setDaemon(true);
		stoping = false;
		error = "";
	}

	public void run() {
		try {
			// deserialisation
			status = "Unserialising";
			run = new Serialising().unserialise_msrun(this.pepnovo_seria
					.getAbsolutePath());
			logger.debug("deserialise");

			// recherche des banques de données
			Vector<File> databases = new Vector<File>();
			if (conf.getDatabase_conta().exists())
				databases.add(conf.getDatabase_conta());
			// ajouter test databases xtandem
			if (run.get_xtandem_prot().size() > 0) {
				msms_output fasta = new msms_output();
				fasta.open(this.get_out("fasta"));
				run.print_fasta_protein(fasta);
				fasta.close();
				databases.add(new File(this.get_out("fasta")));
				logger.debug("Xtandem_fasta");
			}
			if (conf.getDatabase_interest().exists())
				databases.add(conf.getDatabase_interest());
			if (databases.size() == 0)
				throw new PepException("The databases were not found");

			// demarrage de la recherche
			status = "Fasts initialisation";
			fasts_calculation fasts;
			run.filter_pepnovo_sequence(conf.getPepnovo_score());

			for (int i = 0; i < databases.size(); i++) {
				String[] com = { conf.getFasts_exec().getAbsolutePath(), "-Q",
						"-m", "0", "-b", "1", "-d", "1", "-E", "0.001",
						this.get_out("seq"),
						databases.elementAt(i).getAbsolutePath() };
				boolean valid_result = true;
				boolean contain_seq = run.contain_pepnovo_sequence_filter();
				int pass = 0;
				while ((valid_result) & (contain_seq)) {
					fasts = new fasts_calculation();
					fasts.set_Msrun(run);
					fasts.setDatabase(databases.elementAt(i).getName());
					fasts.set_commande(com);
					if (conf.getDatabase_conta().equals(databases.elementAt(i)))
						fasts.setContaminant(true);

					msms_output out = new msms_output();
					out.open(this.get_out("seq"));
					run.print_pepnovo_sequence_filter(out);
					out.close();

					fasts.start_process();
					pass++;
					while (fasts.status()) {
						if (this.stoping) {
							fasts.stop_process();
							throw new StopException("Stop");
						}
						Thread.sleep(1000);
						status = "Databases " + (i + 1) + ":"
								+ databases.size() + " Pass " + pass;
					}
					if (!fasts.get_error().equals(""))
						throw new PepException(fasts.get_error());
					valid_result = fasts.isValid_result();

					// test si il reste de séquence
					contain_seq = run.contain_pepnovo_sequence_filter();
				}
			}

			status = "Save : serialising";
			new Serialising().serialise_msrun(this.get_out("fasts"), run);
			logger.debug("serialise");
			logger.debug("finish");

		} catch (StopException e) {
			logger.debug("Stop");
		} catch (Exception e) {
			this.setError("" + e);
			e.printStackTrace();
		} finally {
			File seq = new File(this.get_out("seq"));
			if (seq.exists())
				seq.delete();
			File fasta = new File(this.get_out("fasta"));
			if (fasta.exists())
				fasta.delete();
		}

	}

	private String get_out(String extension) {
		return (conf.getOutdir().getAbsolutePath()
				+ System.getProperty("file.separator") + basename + "." + extension);
	}

	public void stop_process() {
		stoping = true;
	}

	public void setError(String error) {
		this.error = error;
	}

	public fasts_conf getConf() {
		return conf;
	}

	public void setConf(fasts_conf conf) {
		this.conf = conf;
	}

	public File getPepnovo_seria() {
		return pepnovo_seria;
	}

	public void setPepnovo_seria(File pepnovo_seria) {
		this.pepnovo_seria = pepnovo_seria;
		this.basename = this.pepnovo_seria.getName().replaceAll(".pepnovo", "");
	}

	public Ms_run getRun() {
		return run;
	}

	public void setRun(Ms_run run) {
		this.run = run;
	}

	public String getStatus() {
		return status;
	}

	public String getError() {
		return error;
	}

}
