package fr.inra.pappso.sax_parse;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.apache.log4j.Logger;

import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pipeline.fasts_conf;
import fr.inra.pappso.pipeline.pepnovo_conf;

public class Serialising {
	private static final Logger logger = Logger.getLogger(Serialising.class);

	public Serialising() {

	}

	public void serialise_msrun(String file_out, Ms_run run) throws Exception {
		FileOutputStream fos = null;
		ObjectOutputStream out_ser = null;

		fos = new FileOutputStream(file_out);
		out_ser = new ObjectOutputStream(fos);
		out_ser.writeObject(run);
		run.serialize(out_ser);
		out_ser.close();
	}

	public Ms_run unserialise_msrun(String file_in) throws Exception {

		FileInputStream fis = null;
		ObjectInputStream in = null;

		fis = new FileInputStream(file_in);
		in = new ObjectInputStream(fis);
		Ms_run run = (Ms_run) in.readObject();
		run.unserialize(in);
		in.close();

		return (run);
	}

	public void serialise_pep_conf(pepnovo_conf conf) throws Exception {
		FileOutputStream fos = null;
		ObjectOutputStream out_ser = null;
		String file_out = this.get_conf_save("pepnovo.conf");
		fos = new FileOutputStream(file_out);
		out_ser = new ObjectOutputStream(fos);
		out_ser.writeObject(conf);
		out_ser.close();
	}

	public pepnovo_conf unserialise_pep_conf() throws PepException {

		String file_in = this.get_conf_save("pepnovo.conf");
		File pepnovoConfFile = new File(file_in);
		try {
			FileInputStream fis = null;
			ObjectInputStream in = null;
			if (!pepnovoConfFile.exists()) {
				logger.debug(pepnovoConfFile.getAbsolutePath()
						+ " does not exists");
				return (new pepnovo_conf());
			}
			logger.debug(pepnovoConfFile.getAbsolutePath() + " does exists");

			try {
				fis = new FileInputStream(pepnovoConfFile);
			} catch (FileNotFoundException e1) {
				String message = "pepnovo conf file not found :\n"
						+ pepnovoConfFile.getAbsolutePath();
				logger.error(message);
				throw new PepException(message);
			}
			try {
				in = new ObjectInputStream(fis);
			} catch (IOException e1) {
				fis.close();
				String message = "problem reading pepnovo conf file :\n"
						+ pepnovoConfFile.getAbsolutePath();
				logger.error(message);
				throw new PepException(message);
			}

			pepnovo_conf conf;
			try {
				conf = (pepnovo_conf) in.readObject();
			} catch (ClassNotFoundException e) {
				logger.error("class not found : " + e.getMessage()
						+ " deleting file " + pepnovoConfFile.getAbsolutePath());

				if (pepnovoConfFile.delete()) {
					in.close();
					return new Serialising().unserialise_pep_conf();
				}
				in.close();

				throw new PepException(
						"problem reading "
								+ pepnovoConfFile.getAbsolutePath()
								+ ". This is most likely a file produced by a previous version of the DeNovo pipeline. Please try to remove it from your system et relaunch analysis.");
			} catch (IOException e1) {
				String message = "problem reading pepnovo conf file :\n"
						+ pepnovoConfFile.getAbsolutePath() + "\n"
						+ e1.getMessage();
				logger.error(message);
				throw new PepException(message);
			} catch (Exception e1) {
				String message = "unknown problem reading pepnovo conf file :\n"
						+ pepnovoConfFile.getAbsolutePath()
						+ "\n"
						+ e1.getMessage();
				logger.error(message);
				throw new PepException(message);
			} finally {
				in.close();

			}
			return (conf);
		} catch (PepException e) {
			throw e;
		} catch (IOException e1) {
			String message = "problem reading pepnovo conf file :\n"
					+ pepnovoConfFile.getAbsolutePath() + "\n"
					+ e1.getMessage();
			logger.error(message);
			throw new PepException(message);
		}

	}

	public void serialise_fasts_conf(fasts_conf conf) throws Exception {
		FileOutputStream fos = null;
		ObjectOutputStream out_ser = null;
		String file_out = this.get_conf_save("fasts.conf");
		fos = new FileOutputStream(file_out);
		out_ser = new ObjectOutputStream(fos);
		out_ser.writeObject(conf);
		out_ser.close();
	}

	public fasts_conf unserialise_fasts_conf() throws Exception {

		String file_in = this.get_conf_save("fasts.conf");
		File fastsConfFile = new File(file_in);
		try {
			FileInputStream fis = null;
			ObjectInputStream in = null;

			if (!fastsConfFile.exists()) {
				return (new fasts_conf());
			}

			fis = new FileInputStream(file_in);
			in = new ObjectInputStream(fis);
			try {
				fasts_conf conf = (fasts_conf) in.readObject();
				return (conf);
			} catch (ClassNotFoundException e) {
				logger.error("class not found : " + e.getMessage()
						+ " deleting file " + fastsConfFile.getAbsolutePath());

				if (fastsConfFile.delete()) {
					in.close();
					return new Serialising().unserialise_fasts_conf();
				}
				in.close();

				throw new PepException(
						"problem reading "
								+ fastsConfFile.getAbsolutePath()
								+ ". This is most likely a file produced by a previous version of the DeNovo pipeline. Please try to remove it from your system et relaunch analysis.");
			} catch (IOException e1) {
				String message = "problem reading pepnovo conf file :\n"
						+ fastsConfFile.getAbsolutePath();
				logger.error(message);
				throw new PepException(message);
			} finally {
				in.close();

			}

		} catch (PepException e) {
			throw e;
		} catch (IOException e1) {
			String message = "problem reading pepnovo conf file :\n"
					+ fastsConfFile.getAbsolutePath() + "\n" + e1.getMessage();
			logger.error(message);
			throw new PepException(message);
		}
	}

	private String get_conf_save(String name) {
		String S = System.getProperty("user.home")
				+ System.getProperty("file.separator") + name;
		logger.debug(S);
		return S;
	}

}
